import typing

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from .ViscoElastic.Hysteresis import Hysteresis
from ...TestData.BiaxialTestData import BiaxialTestData
from ...TestData.PlanarTestData import PlanarTestData
from ...TestData.UniaxialTestData import UniaxialTestData
from ...TestData.VolumetricTestData import VolumetricTestData
from ....UtilityAndView.abaqusConstants import *


@abaqus_class_doc
class Hyperelastic:
    r"""The Hyperelastic object specifies elastic properties for approximately incompressible
    elastomers.

    .. note:: 
        This object can be accessed by::

            import material
            mdb.models[name].materials[name].hyperelastic
            import odbMaterial
            session.odbs[name].materials[name].hyperelastic

        The table data for this object are:

        - If **type** = ARRUDA_BOYCE, the table data specify the following:
        
            - :math:`\mu`.
            - :math:`\lambda_{m}`.
            - :math:`D`.
            - Temperature, if the data depend on temperature.
            
        - If **type** = MOONEY_RIVLIN, the table data specify the following:
        
            - :math:`C_{10}`
            - :math:`C_{01}`.
            - :math:`D_{1}`.
            - Temperature, if the data depend on temperature.
            
        - If **type** = NEO_HOOKE, the table data specify the following:
        
            - :math:`C_{10}`
            - :math:`D_{1}`.
            - Temperature, if the data depend on temperature.
            
        - If **type** = OGDEN, the table data specify the following for values of nn:
        
            - :math:`\mu_{i}` and :math:`\alpha_{i}` for :math:`i` from 1 to :math:`n`.
            - :math:`n` coefficients :math:`D_{i}`.
            - Temperature, if the data depend on temperature. Temperature dependence is not 
              allowed for :math:`4 \leq n \leq 6` in an Abaqus/Explicit analysis.
              
        - If **type** = POLYNOMIAL, the table data specify the following for values of nn:
        
            - :math:`C_{i j}` for each value of :math:`(i+j)` from 1 to :math:`n` with 
              :math:`i` decreasing from :math:`(i+j)` to zero and :math:`j` increasing 
              from zero to :math:`(i+j)`.
            - :math:`n` coefficients :math:`D_{i}`
            - Temperature, if the data depend on temperature. Temperature dependence is
              not allowed for :math:`3 \leq n \leq 6` in an Abaqus/Explicit analysis.
        
        - If **type** = REDUCED_POLYNOMIAL, the table data specify the following for values of nn:
        
            - :math:`C_{i 0}` for :math:`i` from 1 to :math:`n`.
            - :math:`n` coefficients :math:`D_{i}`
            - Temperature, if the data depend on temperature. Temperature dependence 
              is not allowed for :math:`4 \leq n \leq 6` in an Abaqus/Explicit analysis.
              
        - If **type** = VAN_DER_WAALS, the table data specify the following:
        
            - :math:`\mu`.
            - :math:`\lambda_{m}`.
            - :math:`a`.
            - :math:`\beta`.
            - :math:`D`.
            - Temperature, if the data depend on temperature.
            
        - If **type** = YEOH, the table data specify the following:
        
            - :math:`C_{10}`
            - :math:`C_{20}`
            - :math:`C_{30}`
            - :math:`D_{1}`.
            - :math:`D_{2}`
            - :math:`D_{3}`.
            - Temperature, if the data depend on temperature. Temperature dependence 
              is not allowed in an Abaqus/Explicit analysis.
        
        The None object is the default value if **testData** = ON.

        The corresponding analysis keywords are:

        - HYPERELASTIC
    """

    #: A :py:class:`~abaqus.Material.TestData.BiaxialTestData.BiaxialTestData` object.
    biaxialTestData: BiaxialTestData = BiaxialTestData(((),))

    #: A :py:class:`~abaqus.Material.TestData.PlanarTestData.PlanarTestData` object.
    planarTestData: PlanarTestData = PlanarTestData(((),))

    #: A :py:class:`~abaqus.Material.TestData.UniaxialTestData.UniaxialTestData` object.
    uniaxialTestData: UniaxialTestData = UniaxialTestData(((),))

    #: A :py:class:`~abaqus.Material.TestData.VolumetricTestData.VolumetricTestData` object.
    volumetricTestData: VolumetricTestData = VolumetricTestData(((),))

    #: A :py:class:`~abaqus.Material.Elastic.HyperElastic.ViscoElastic.Hysteresis.Hysteresis` object.
    hysteresis: Hysteresis = Hysteresis(((),))

    @abaqus_method_doc
    def __init__(
        self,
        table: tuple,
        type: SymbolicConstant = UNKNOWN,
        moduliTimeScale: SymbolicConstant = LONG_TERM,
        temperatureDependency: Boolean = OFF,
        n: int = 1,
        beta: typing.Union[SymbolicConstant, float] = FITTED_VALUE,
        testData: Boolean = ON,
        compressible: Boolean = OFF,
        properties: int = 0,
        deviatoricResponse: SymbolicConstant = UNIAXIAL,
        volumetricResponse: SymbolicConstant = DEFAULT,
        poissonRatio: float = 0,
        materialType: SymbolicConstant = ISOTROPIC,
        anisotropicType: SymbolicConstant = FUNG_ANISOTROPIC,
        formulation: SymbolicConstant = STRAIN,
        behaviorType: SymbolicConstant = INCOMPRESSIBLE,
        dependencies: int = 0,
        localDirections: int = 0,
    ):
        """This method creates a Hyperelastic object.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].materials[name].Hyperelastic
                session.odbs[name].materials[name].Hyperelastic

        Parameters
        ----------
        table
            A sequence of sequences of Floats specifying the items described below. This argument is
            valid only if **testData** = OFF.
        type
            A SymbolicConstant specifying the type of strain energy potential. Possible values
            are:

            - ARRUDA_BOYCE
            - MARLOW
            - MOONEY_RIVLIN
            - NEO_HOOKE
            - OGDEN
            - POLYNOMIAL
            - REDUCED_POLYNOMIAL
            - USER
            - VAN_DER_WAALS
            - YEOH
            - UNKNOWN

            The default value is UNKNOWN.
        moduliTimeScale
            A SymbolicConstant specifying the nature of the time response. Possible values are
            INSTANTANEOUS and LONG_TERM. The default value is LONG_TERM.
        temperatureDependency
            A Boolean specifying whether the data depend on temperature. The default value is OFF.
        n
            An Int specifying the order of the strain energy potential. The default value is 1.If
            **testData** = ON and **type** = POLYNOMIAL, **n** can take only the values 1 or 2.If
            **testData** = ON and **type** = OGDEN or if **testData** = OFF for either type, 1 ≤n≤n≤ 6.If
            **type** = USER, this argument cannot be used.
        beta
            The SymbolicConstant FITTED_VALUE or a Float specifying the invariant mixture parameter.
            This argument is valid only if **testData** = ON and **type** = VAN_DER_WAALS. The default value
            is FITTED_VALUE.
        testData
            A Boolean specifying whether test data are supplied. The default value is ON.
        compressible
            A Boolean specifying whether the hyperelastic material is compressible. This parameter
            is applicable only to user-defined hyperelastic materials. The default value is OFF.
        properties
            An Int specifying the number of property values needed as data for the user-defined
            hyperelastic material. The default value is 0.
        deviatoricResponse
            A SymbolicConstant specifying which test data to use. Possible values are UNIAXIAL,
            BIAXIAL, and PLANAR. The default value is UNIAXIAL.
        volumetricResponse
            A SymbolicConstant specifying the volumetric response. Possible values are DEFAULT,
            VOLUMETRIC_DATA, POISSON_RATIO, and LATERAL_NOMINAL_STRAIN. The default value is
            DEFAULT.
        poissonRatio
            A Float specifying the poisson ratio. This argument is valid only if
            **volumetricResponse** = POISSON_RATIO. The default value is 0.0.
        materialType
            A SymbolicConstant specifying the type of material. Possible values are ISOTROPIC and
            ANISOTROPIC. The default value is ISOTROPIC.
        anisotropicType
            A SymbolicConstant specifying the type of strain energy potential. Possible values are
            FUNG_ANISOTROPIC, FUNG_ORTHOTROPIC, HOLZAPFEL, and USER_DEFINED. The default value is
            FUNG_ANISOTROPIC.
        formulation
            A SymbolicConstant specifying the type of formulation. Possible values are STRAIN and
            INVARIANT. The default value is STRAIN.
        behaviorType
            A SymbolicConstant specifying the type of anisotropic hyperelastic material behavior.
            Possible values are INCOMPRESSIBLE and COMPRESSIBLE. The default value is
            INCOMPRESSIBLE.
        dependencies
            An Int specifying the number of field variable dependencies for the data
            in*volumetricTable* . The default value is 0.
        localDirections
            An Int specifying the number of local directions for the data in*volumetricTable* . The
            default value is 0.

        Returns
        -------
        Hyperelastic
            A :py:class:`~abaqus.Material.Elastic.HyperElastic.Hyperelastic.Hyperelastic` object.

        Raises
        ------
        InvalidNameError
        RangeError
        """
        ...

    @abaqus_method_doc
    def setValues(self, *args, **kwargs):
        """This method modifies the Hyperelastic object.

        Raises
        ------
        RangeError
        """
        ...



import typing

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from .ConstrainedSketchGeometry import ConstrainedSketchGeometry


@abaqus_class_doc
class ConstructionLine(ConstrainedSketchGeometry):
    @abaqus_method_doc
    def __init__(self, point1: typing.Tuple[float, ...], point2: typing.Tuple[float, ...]):
        """This method creates an oblique construction line that runs between two given points.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].sketches[name].ConstructionLine

        Parameters
        ----------
        point1
            A pair of Floats specifying the first endpoint.
        point2
            A pair of Floats specifying the second endpoint.

        Returns
        -------
        ConstrainedSketchGeometry
            A :py:class:`~abaqus.Sketcher.ConstrainedSketchGeometry.ConstrainedSketchGeometry.ConstrainedSketchGeometry` object (None if the line cannot be created).

        """
        ...



from abqpy.decorators import abaqus_class_doc
from .CommonOptions import CommonOptions
from .ContourOptions import ContourOptions
from .DisplayBodyOptions import DisplayBodyOptions
from .OrientationOptions import OrientationOptions
from .SuperimposeOptions import SuperimposeOptions
from .SymbolOptions import SymbolOptions
from ..PlotOptions.BasicOptions import BasicOptions
from ..PlotOptions.FreeBodyOptions import FreeBodyOptions
from ..PlotOptions.StreamOptions import StreamOptions
from ..PlotOptions.ViewCutOptions import ViewCutOptions


@abaqus_class_doc
class DefaultOdbDisplay:
    """The DefaultOdbDisplay object is a limited-functionality version of the OdbDisplay
    object.

    .. note:: 
        This object can be accessed by::

            import visualization
            session.defaultOdbDisplay
    """

    #: A :py:class:`~abaqus.PlotOptions.BasicOptions.BasicOptions` object.
    basicOptions: BasicOptions = BasicOptions()

    #: A :py:class:`~abaqus.OdbDisplay.CommonOptions.CommonOptions` object.
    commonOptions: CommonOptions = CommonOptions()

    #: A :py:class:`~abaqus.OdbDisplay.ContourOptions.ContourOptions` object.
    contourOptions: ContourOptions = ContourOptions()

    #: A :py:class:`~abaqus.OdbDisplay.DisplayBodyOptions.DisplayBodyOptions` object.
    displayBodyOptions: DisplayBodyOptions = DisplayBodyOptions()

    #: A :py:class:`~abaqus.PlotOptions.FreeBodyOptions.FreeBodyOptions` object.
    freeBodyOptions: FreeBodyOptions = FreeBodyOptions()

    #: A :py:class:`~abaqus.PlotOptions.StreamOptions.StreamOptions` object.
    streamOptions: StreamOptions = StreamOptions()

    #: An :py:class:`~abaqus.OdbDisplay.OrientationOptions.OrientationOptions` object.
    materialOrientationOptions: OrientationOptions = OrientationOptions()

    #: A :py:class:`~abaqus.OdbDisplay.SuperimposeOptions.SuperimposeOptions` object.
    superimposeOptions: SuperimposeOptions = SuperimposeOptions()

    #: A :py:class:`~abaqus.OdbDisplay.SymbolOptions.SymbolOptions` object.
    symbolOptions: SymbolOptions = SymbolOptions()

    #: A :py:class:`~abaqus.PlotOptions.ViewCutOptions.ViewCutOptions` object.
    viewCutOptions: ViewCutOptions = ViewCutOptions()



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from ..UtilityAndView.abaqusConstants import *


@abaqus_class_doc
class NetworkDatabaseConnector:
    """The NetworkDatabaseConnector object allows you to access an output database on a remote
    system.

    .. note:: 
        This object can be accessed by::

            session.networkDatabaseConnectors[name]
    """

    #: A Boolean specifying if the connection between the client and the server is established.
    connected: Boolean = OFF

    #: A String specifying the repository key.
    name: str

    #: A String specifying the name of the remote computer.
    hostName: str

    #: A String specifying the directory on the remote computer.
    directory: str

    #: A String specifying the name of command to execute Abaqus/CAE on the remote computer.
    remoteAbaqusDriverName: str = ""

    #: A SymbolicConstant specifying the remote shell command on the local system. Possible
    #: values are RSH and SSH. The default value is SSH.
    remoteLoginMechanism: SymbolicConstant = SSH

    #: A String specifying the path to the`ssh` command on the local system. The default value
    #: is an empty string.
    sshPath: str = ""

    #: An Int specifying the server port on the remote computer. If **serverPort** =0, the host
    #: and remote systems are allowed to establish their own port numbers. The default value is
    #: 0.
    serverPort: int = 0

    #: An Int specifying the connection port on the remote computer. The default value is 0.
    connectionPort: int = 0

    #: An Int specifying the timeout in seconds for the remote server. For example: 86400
    #: corresponds to one day. The server exits if it does not receive any communication from
    #: the client during the time specified. The default value is 86400.
    serverTimeout: int = 86400

    #: A Boolean specifying whether to start the remote network database connector server. The
    #: default value is ON.
    allowAutomaticStartup: Boolean = ON

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        hostName: str,
        directory: str,
        remoteAbaqusDriverName: str = "",
        remoteLoginMechanism: SymbolicConstant = SSH,
        sshPath: str = "",
        serverPort: int = 0,
        connectionPort: int = 0,
        serverTimeout: int = 86400,
        allowAutomaticStartup: Boolean = ON,
    ):
        """This method creates a NetworkDatabaseConnector object that you can use to access a
        remote output database. You can create a network database connector from any platform:
        Windows or Linux. However, the network database connector server must reside on a Linux
        platform; you cannot access an output database that resides on a remote Windows system.
        You can access only a remote output database; you cannot access a remote model database.

        .. note:: 
            This function can be accessed by::

                session.NetworkDatabaseConnector

        Parameters
        ----------
        name
            A String specifying the repository key.
        hostName
            A String specifying the name of the remote computer.
        directory
            A String specifying the directory on the remote computer.
        remoteAbaqusDriverName
            A String specifying the name of command to execute Abaqus/CAE on the remote computer.
        remoteLoginMechanism
            A SymbolicConstant specifying the remote shell command on the local system. Possible
            values are RSH and SSH. The default value is SSH.
        sshPath
            A String specifying the path to the`ssh` command on the local system. The default value
            is an empty string.
        serverPort
            An Int specifying the server port on the remote computer. If **serverPort** =0, the host
            and remote systems are allowed to establish their own port numbers. The default value is
            0.
        connectionPort
            An Int specifying the connection port on the remote computer. The default value is 0.
        serverTimeout
            An Int specifying the timeout in seconds for the remote server. For example: 86400
            corresponds to one day. The server exits if it does not receive any communication from
            the client during the time specified. The default value is 86400.
        allowAutomaticStartup
            A Boolean specifying whether to start the remote network database connector server. The
            default value is ON.

        Returns
        -------
        NetworkDatabaseConnector
            A :py:class:`~abaqus.Session.NetworkDatabaseConnector.NetworkDatabaseConnector` object.
        """
        ...

    @abaqus_method_doc
    def start(self, serverPort: int = 0, serverTimeout: int = 86400):
        """This method starts the remote network database connector server on the remote host.

        Parameters
        ----------
        serverPort
            An Int specifying the server port on the remote computer. If **serverPort** =0, the host
            and remote systems are allowed to establish their own port numbers. The default value is
            0.
        serverTimeout
            An Int specifying the timeout in seconds for the remote server. For example: 86400
            corresponds to one day. The server exits if it does not receive any communication from
            the client during the time specified. The default value is 86400.
        """
        ...

    @abaqus_method_doc
    def stop(self):
        """This method stops the remote network database connector server on the remote host."""
        ...

    @abaqus_method_doc
    def setValues(
        self,
        remoteAbaqusDriverName: str = "",
        remoteLoginMechanism: SymbolicConstant = SSH,
        sshPath: str = "",
        serverPort: int = 0,
        connectionPort: int = 0,
        serverTimeout: int = 86400,
        allowAutomaticStartup: Boolean = ON,
    ):
        """This method modifies the NetworkDatabaseConnector object.

        Parameters
        ----------
        remoteAbaqusDriverName
            A String specifying the name of command to execute Abaqus/CAE on the remote computer.
        remoteLoginMechanism
            A SymbolicConstant specifying the remote shell command on the local system. Possible
            values are RSH and SSH. The default value is SSH.
        sshPath
            A String specifying the path to the`ssh` command on the local system. The default value
            is an empty string.
        serverPort
            An Int specifying the server port on the remote computer. If **serverPort** =0, the host
            and remote systems are allowed to establish their own port numbers. The default value is
            0.
        connectionPort
            An Int specifying the connection port on the remote computer. The default value is 0.
        serverTimeout
            An Int specifying the timeout in seconds for the remote server. For example: 86400
            corresponds to one day. The server exits if it does not receive any communication from
            the client during the time specified. The default value is 86400.
        allowAutomaticStartup
            A Boolean specifying whether to start the remote network database connector server. The
            default value is ON.
        """
        ...



import re
import typing
from . import __version__ as version

version = version[:4]


class AbaqusDoc:

    @classmethod
    def _class_or_module_link(
        cls,
        type: str,
        class_or_module_name: str,
        prefix='',
        suffix='',
        label=None,
    ) -> typing.Tuple[str, str]:
        """Generate a link to the Abaqus class documentation.

        Parameters
        ----------
        type : 'class' or 'module'
            Type of the object, 'class' or 'module'.
        class_or_module_name : str
            The name of the class or module.
        prefix : str
            The prefix to the class or module name.
        suffix : str
            The suffix to the class name.
        label : str
            The label to use for the link.

        Returns
        -------
        str
            The link to the documentation.
        str
            The link with label.
        """
        if label is None:
            label = class_or_module_name
        module_name = f"{type if type == 'module' else ''}{class_or_module_name.lower()}"
        link = (f"https://help.3ds.com/{version}/English/DSSIMULIA_Established/SIMACAEKERRefMap/"
                f"simaker-c-{prefix}{module_name}{suffix}pyc.htm?contextscope=all")
        return link, f"Check `{label} on help.3ds.com/{version} <{link}>`__."

    @classmethod
    def _method_or_function_link(
        cls,
        type: str,
        class_or_module_name: str,
        method_or_function_name: str,
        prefix='',
        suffix='',
        label=None,
    ) -> typing.Tuple[str, str]:
        """Generate a link to the Abaqus function documentation.

        Parameters
        ----------
        type : 'method' or 'function'
            Type of the object, 'method' or 'function'.
        class_or_module_name : str
            The name of the module.
        method_or_function_name : str
            The name of the function.
        prefix : str
            The prefix to the class or module name.
        suffix : str
            The suffix to the class name.
        label : str
            The label to use for the link.

        Returns
        -------
        str
            The link to the documentation.
        str
            The link with label.
        """
        # For methods that start with a capital letter, the link is to the corresponding class documentation.
        if method_or_function_name[0].isupper():
            return cls._class_or_module_link('class', method_or_function_name, prefix, suffix, method_or_function_name)
        class_name = f"{type if type == 'function' else ''}{class_or_module_name.lower()}"
        function_prefix = prefix if type == 'function' else ''
        if type == 'class' and method_or_function_name == '__init__':
            method_or_function_name = class_or_module_name
        link = (f"https://help.3ds.com/{version}/English/DSSIMULIA_Established/SIMACAEKERRefMap/"
                f"simaker-c-{prefix}{class_name}{suffix}pyc.htm?contextscope=all"
                f"#simaker-{function_prefix}{class_name}{method_or_function_name.lower()}pyc")
        if method_or_function_name == class_or_module_name and type == 'class':
            signature = f"{class_or_module_name}()"
        else:
            signature = f"{class_or_module_name}.{method_or_function_name}()"
        if label is None:
            label = signature
        return link, f"Check `{label} on help.3ds.com/{version} <{link}>`__."

    @classmethod
    def _add_link_in_class_or_module_docstring(
        cls,
        type: str,
        class_or_module_name: str,
        docstring: str,
        prefix='',
        suffix='',
        label=None,
    ) -> str:
        """Add a link to the Abaqus documentation to the docstring for classes or modules.

        Parameters
        ----------
        type : 'class' or 'module'
            Type of the object, 'class' or 'module'.
        class_or_module_name : str
            The name of the class or module.
        docstring : str
            The docstring.
        prefix : str
            The prefix to the class or module name.
        suffix : str
            The suffix to the class name.
        label : str
            The label to use for the link.

        Returns
        -------
        str
            The docstring with the link to the Abaqus documentation.
        """
        if not docstring:
            return docstring
        link, link_with_label = cls._class_or_module_link(type, class_or_module_name, prefix, suffix, label)
        return (docstring.rstrip() + '\n\n' + ' ' * (0 if type == 'module' else 4) +
                '.. note::\n' + ' ' * (4 if type == 'module' else 8) + link_with_label)

    @classmethod
    def _add_link_in_method_or_function_docstring(
        cls,
        type: str,
        class_or_module_name: str,
        method_or_function_name: str,
        docstring: str,
        prefix='',
        suffix='',
        label=None,
    ) -> str:
        """Add a link to the Abaqus documentation to the docstring for methods or functions.

        Parameters
        ----------
        type : 'method' or 'function'
            Type of the object, 'method' or 'function'.
        class_or_module_name : str
            The name of the module.
        method_or_function_name : str
            The name of the function.
        docstring : str
            The docstring.
        prefix : str
            The prefix to the class or module name.
        suffix : str
            The suffix to the class name.
        label : str
            The label to use for the link.

        Returns
        -------
        str
            The docstring with the link to the Abaqus documentation.
        """
        if not docstring:
            return docstring
        link, link_with_label = cls._method_or_function_link(type, class_or_module_name,
                                                             method_or_function_name, prefix, suffix, label)
        return re.sub(r'(\n\s+?)(Parameters\n\s+----------)',
                      r'\1' + '.. note::\n' + ' ' * (8 if type == 'function' else 12) + link_with_label + r'\1\2',
                      docstring)

    @classmethod
    def add_link_in_class_docstring(
        cls,
        class_name: str,
        docstring: str,
        prefix='',
        suffix='',
        label=None,
    ) -> str:
        """Add a link to the Abaqus documentation to the docstring for classes.

        Parameters
        ----------
        class_name : str
            The name of the class.
        docstring : str
            The docstring.
        prefix : str
            The prefix to the class or module name.
        suffix : str
            The suffix to the class name.
        label : str
            The label to use for the link.

        Returns
        -------
        str
            The docstring with the link to the Abaqus documentation.
        """
        return cls._add_link_in_class_or_module_docstring(
            'class', class_name, docstring, prefix, suffix, label
        )

    @classmethod
    def add_link_in_module_docstring(
        cls,
        module_name: str,
        docstring: str,
        prefix='',
        suffix='',
        label=None,
    ) -> str:
        """Add a link to the Abaqus documentation to the docstring for modules.

        Parameters
        ----------
        module_name : str
            The name of the module.
        docstring : str
            The docstring.
        prefix : str
            The prefix to the class or module name.
        suffix : str
            The suffix to the class name.
        label : str
            The label to use for the link.

        Returns
        -------
        str
            The docstring with the link to the Abaqus documentation.
        """
        return cls._add_link_in_class_or_module_docstring(
            'module', module_name, docstring, prefix, suffix, label
        )

    @classmethod
    def add_link_in_function_docstring(
        cls,
        module_name: str,
        function_name: str,
        docstring: str,
        prefix='',
        suffix='',
        label=None,
    ) -> str:
        """Add a link to the Abaqus documentation to the docstring for functions.

        Parameters
        ----------
        module_name : str
            The name of the module.
        function_name : str
            The name of the function.
        docstring : str
            The docstring.
        prefix : str
            The prefix to the class or module name.
        suffix : str
            The suffix to the class name.
        label : str
            The label to use for the link.

        Returns
        -------
        str
            The docstring with the link to the Abaqus documentation.
        """
        return cls._add_link_in_method_or_function_docstring(
            'function', module_name, function_name, docstring, prefix, suffix, label
        )

    @classmethod
    def add_link_in_method_docstring(
        cls,
        class_name: str,
        method_name: str,
        docstring: str,
        prefix='',
        suffix='',
        label=None,
    ) -> str:
        """Add a link to the Abaqus documentation to the docstring for methods.

        Parameters
        ----------
        class_name : str
            The name of the class.
        method_name : str
            The name of the method.
        docstring : str
            The docstring.
        prefix : str
            The prefix to the class or module name.
        suffix : str
            The suffix to the class name.
        label : str
            The label to use for the link.

        Returns
        -------
        str
            The docstring with the link to the Abaqus documentation.
        """
        return cls._add_link_in_method_or_function_docstring(
            'method', class_name, method_name, docstring, prefix, suffix, label
        )


doc = AbaqusDoc


def abaqus_function_doc(func):
    """Add a link to the Abaqus documentation to the docstring of the function.
    """
    module_name = func.__module__.split('.')[-1]
    func.__doc__ = doc.add_link_in_function_docstring(
        module_name=module_name,
        function_name=func.__name__,
        docstring=func.__doc__,
        prefix='gpr' if module_name.lower().startswith('cae') else '',
        label=f'{module_name}.{func.__name__}',
    )
    return func


def _process_class_name(class_name: str) -> str:
    """Process the class name.
    """
    # Handle the base class case.
    if class_name.endswith('Base'):
        class_name = class_name[:-4]
    # Handle the class with a suffix.
    if class_name.endswith('Mdb'):
        class_name = 'Mdb'
    if class_name.endswith('Model'):
        class_name = 'Model'
    if class_name.endswith('Step'):
        class_name = 'Step'
    if class_name.endswith('Assembly'):
        class_name = 'Assembly'
    if class_name.endswith('Step'):
        class_name = 'Step'
    if class_name.endswith('Odb'):
        class_name = 'Odb'
    if class_name.endswith('Part'):
        class_name = 'Part'
    return class_name


class_suffix = {
    'AdaptivityModel': 'kar',
    'AssemblyModel': 'asm',
    'EditMeshAssembly': 'edt',
    'EditMeshPart': 'edt',
    'InteractionModel': 'itn',
    'MeshAssembly': 'mgn',
    'MeshPart': 'mgn',
    'RegionAssembly': 'reg',
    'RegionPart': 'reg',
}


def abaqus_method_doc(method):
    """Add a link to the Abaqus documentation to the docstring of the method.
    """
    class_name = method.__qualname__.split('.')[0]
    class_name = _process_class_name(class_name)
    method.__doc__ = doc.add_link_in_method_docstring(
        class_name=_process_class_name(class_name),
        method_name=method.__name__,
        docstring=method.__doc__,
        prefix='gpr' if class_name.lower().startswith('cae') else '',
        suffix=class_suffix.get(class_name, ''),
        label=f'{class_name}.{method.__name__}',
    )
    return method


def abaqus_class_doc(cls):
    """Add a link to the Abaqus documentation to the docstring of the class.
    """
    class_name = cls.__name__
    processed_class_name = _process_class_name(class_name)
    cls.__doc__ = doc.add_link_in_class_docstring(
        class_name=processed_class_name,
        docstring=cls.__doc__,
        prefix='gpr' if class_name.lower().startswith('cae') else '',
        suffix=class_suffix.get(class_name, ''),
        label=class_name,
    )
    return cls


@abaqus_function_doc
def foo(a, b):
    """This is a test function.

    Parameters
    ----------
    a : int
        The first parameter.
    b : int
        The second parameter.
    """
    return a + b


@abaqus_class_doc
class Bar:
    """This is a test class.
    """

    @abaqus_method_doc
    def foo(self, a, b):
        """This is a test method.

        Parameters
        ----------
        a : int
            The first parameter.
        b : int
            The second parameter.
        """
        pass


def test_foo_bar():
    """
    Test the foo function.
    """
    # foo
    print('\n')
    print(foo.__name__)
    print(foo.__doc__)

    # Bar
    print(Bar.__name__)
    print(Bar.__doc__)

    # Bar.foo
    print(Bar.foo.__name__)
    print(Bar.foo.__doc__)



from abqpy.decorators import abaqus_class_doc
from .StructuralDampingByFrequencyComponentArray import (
    StructuralDampingByFrequencyComponentArray,
)


@abaqus_class_doc
class StructuralDampingByFrequency:
    """A :py:class:`~abaqus.StepMiscellaneous.StructuralDampingByFrequency.StructuralDampingByFrequency` object contains structural damping parameters.

    .. note:: 
        This object can be accessed by::

            import step
            mdb.models[name].steps[name].structuralDampingByFrequency
    """

    #: A :py:class:`~abaqus.StepMiscellaneous.StructuralDampingByFrequencyComponentArray.StructuralDampingByFrequencyComponentArray` object.
    components: StructuralDampingByFrequencyComponentArray = []



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from .AcousticImpedanceProp import AcousticImpedanceProp
from .ActuatorSensorProp import ActuatorSensorProp
from .CavityRadiationProp import CavityRadiationProp
from .ContactProperty import ContactProperty
from .FilmConditionProp import FilmConditionProp
from .FluidCavityProperty import FluidCavityProperty
from .FluidExchangeProperty import FluidExchangeProperty
from .FluidInflatorProperty import FluidInflatorProperty
from .IncidentWaveProperty import IncidentWaveProperty
from ..Model.ModelBase import ModelBase
from ..UtilityAndView.abaqusConstants import *


@abaqus_class_doc
class InteractionPropertyModel(ModelBase):
    @abaqus_method_doc
    def AcousticImpedanceProp(
        self,
        name: str,
        tableType: SymbolicConstant,
        table: tuple,
        frequencyDependency: Boolean = OFF,
    ) -> AcousticImpedanceProp:
        """This method creates an AcousticImpedanceProp object.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].AcousticImpedanceProp

        Parameters
        ----------
        name
            A String specifying the interaction property repository key.
        tableType
            A SymbolicConstant specifying the type of tabular data to be defined. Possible values
            are IMPEDANCE and ADMITTANCE.
        table
            A sequence of sequences of Floats specifying acoustic impedance properties.If
            **tableType** = IMPEDANCE, each sequence of the table data specifies:The real part of the
            complex impedance.The imaginary part of the complex impedance.Frequency, if the data
            depend on frequency.If **tableType** = ADMITTANCE, each sequence of the table data
            specifies:The real part of the complex admittance.The imaginary part of the complex
            admittance.Frequency, if the data depend on frequency.
        frequencyDependency
            A Boolean specifying whether the **table** data depend on frequency. The default value is
            OFF.

        Returns
        -------
        AcousticImpedanceProp
            An :py:class:`~abaqus.Interaction.AcousticImpedanceProp.AcousticImpedanceProp` object.
        """
        self.interactionProperties[name] = interactionProperty = AcousticImpedanceProp(
            name, tableType, table, frequencyDependency
        )
        return interactionProperty

    @abaqus_method_doc
    def ActuatorSensorProp(
        self, name: str, realProperties: tuple = (), integerProperties: tuple = ()
    ) -> ActuatorSensorProp:
        """This method creates an ActuatorSensorProp object.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].ActuatorSensorProp

        Parameters
        ----------
        name
            A String specifying the interaction property repository key.
        realProperties
            A sequence of Floats specifying the PROPS array used by user subroutine UEL. The default
            value is an empty sequence.
        integerProperties
            A sequence of Ints specifying the JPROPS array used by user subroutine UEL. The default
            value is an empty sequence.

        Returns
        -------
        ActuatorSensorProp
            An :py:class:`~abaqus.Interaction.ActuatorSensorProp.ActuatorSensorProp` object.
        """
        self.interactionProperties[name] = interactionProperty = ActuatorSensorProp(
            name, realProperties, integerProperties
        )
        return interactionProperty

    @abaqus_method_doc
    def CavityRadiationProp(
        self,
        name: str,
        temperatureDependency: Boolean = OFF,
        dependencies: int = 0,
        property: tuple = (),
    ) -> CavityRadiationProp:
        """This method creates a CavityRadiationProp object.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].CavityRadiationProp

        Parameters
        ----------
        name
            A String specifying the interaction property repository key.
        temperatureDependency
            A Boolean specifying whether the data depend on temperature. The default value is OFF.
        dependencies
            An Int specifying the number of field variable dependencies. The default value is 0.
        property
            A sequence of sequences of Floats specifying the following:The emissivity,
            ϵϵ.Temperature, if the data depend on temperature.Value of the first field variable, if
            the data depend on field variables.Value of the second field variable.Etc.

        Returns
        -------
        CavityRadiationProp
            A :py:class:`~abaqus.Interaction.CavityRadiationProp.CavityRadiationProp` object.
        """
        self.interactionProperties[name] = interactionProperty = CavityRadiationProp(
            name, temperatureDependency, dependencies, property
        )
        return interactionProperty

    @abaqus_method_doc
    def ContactProperty(self, name: str) -> ContactProperty:
        """This method creates a ContactProperty object.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].ContactProperty

        Parameters
        ----------
        name
            A String specifying the interaction property repository key.

        Returns
        -------
        ContactProperty
            A :py:class:`~abaqus.Interaction.ContactProperty.ContactProperty` object.
        """
        self.interactionProperties[name] = interactionProperty = ContactProperty(name)
        return interactionProperty

    @abaqus_method_doc
    def FilmConditionProp(
        self,
        name: str,
        temperatureDependency: Boolean = OFF,
        dependencies: int = 0,
        property: tuple = (),
    ) -> FilmConditionProp:
        """This method creates a FilmConditionProp object.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].FilmConditionProp

        Parameters
        ----------
        name
            A String specifying the interaction property repository key.
        temperatureDependency
            A Boolean specifying whether the data depend on temperature. The default value is OFF.
        dependencies
            An Int specifying the number of field variable dependencies. The default value is 0.
        property
            A sequence of sequences of Floats specifying the following:
            
            - The film coefficient, hh.
            - Temperature, if the data depend on temperature.
            - Value of the first field variable, if the data depend on field variables.
            - Value of the second field variable.
            - Etc.

        Returns
        -------
        FilmConditionProp
            A :py:class:`~abaqus.Interaction.FilmConditionProp.FilmConditionProp` object.
        """
        self.interactionProperties[name] = interactionProperty = FilmConditionProp(
            name, temperatureDependency, dependencies, property
        )
        return interactionProperty

    @abaqus_method_doc
    def FluidCavityProperty(
        self,
        name: str,
        definition: SymbolicConstant = HYDRAULIC,
        fluidDensity: float = None,
        molecularWeight: float = None,
        useExpansion: Boolean = OFF,
        expansionTempDep: Boolean = OFF,
        expansionDependencies: int = 0,
        referenceTemperature: float = 0,
        expansionTable: tuple = (),
        useBulkModulus: Boolean = OFF,
        bulkModulusTempDep: Boolean = OFF,
        bulkModulusDependencies: int = 0,
        bulkModulusTable: tuple = (),
        useCapacity: Boolean = OFF,
        capacityType: SymbolicConstant = POLYNOMIAL,
        capacityTempDep: Boolean = OFF,
        capacityDependencies: int = 0,
        capacityTable: tuple = (),
    ) -> FluidCavityProperty:
        """This method creates a FluidCavityProperty object.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].FluidCavityProperty

        Parameters
        ----------
        name
            A String specifying the interaction property repository key.
        definition
            A SymbolicConstant specifying the type of fluid cavity property to be defined. Possible
            values are HYDRAULIC and PNEUMATIC. The default value is HYDRAULIC.
        fluidDensity
            None or a Float specifying the reference fluid density. This argument is applicable only
            when **definition** = HYDRAULIC, and is required in that case. The default value is None.
        molecularWeight
            None or a Float specifying the molecular weight of the ideal gas species. This argument
            is applicable only when **definition** = PNEUMATIC, and is required in that case. The
            default value is None.
        useExpansion
            A Boolean specifying whether thermal expansion coefficients will be defined. This
            argument is applicable only when **definition** = HYDRAULIC. The default value is OFF.
        expansionTempDep
            A Boolean specifying whether the thermal fluid expansion data will have temperature
            dependency. This argument is applicable only when **definition** = HYDRAULIC and when
            **useExpansion** = True. The default value is OFF.
        expansionDependencies
            An Int specifying the number of field variable dependencies in the thermal fluid
            expansion data. This argument is applicable only when **definition** = HYDRAULIC and when
            **useExpansion** = True. The default value is 0.
        referenceTemperature
            A Float specifying the reference temperature for the coefficient of thermal expansion.
            This argument is applicable only when **definition** = HYDRAULIC, when **useExpansion** = True,
            and when either **expansionTempDep** = True or when **expansionDependencies** is greater than
            0. The default value is 0.0.
        expansionTable
            A sequence of sequences of Floats specifying the thermal expansion coefficients. This
            argument is applicable only when **definition** = HYDRAULIC and when **useExpansion** = True.
            Each sequence contains the following data:
            
            - The mean coefficient of thermal expansion.
            - Temperature, if the data depend on temperature.
            - Value of the first field variable, if the data depend on field variables.
            - Value of the second field variable.
            - Etc.
        useBulkModulus
            A Boolean specifying whether fluid bulk modulus values will be defined. This argument is
            applicable only when **definition** = HYDRAULIC. The default value is OFF.
        bulkModulusTempDep
            A Boolean specifying whether the fluid bulk modulus data will have temperature
            dependency. This argument is applicable only when **definition** = HYDRAULIC and when
            **useBulkModulus** = True. The default value is OFF.
        bulkModulusDependencies
            An Int specifying the number of field variable dependencies in the fluid bulk modulus
            data. This argument is applicable only when **definition** = HYDRAULIC and when
            **useBulkModulus** = True. The default value is 0.
        bulkModulusTable
            A sequence of sequences of Floats specifying the fluid bulk modulus values. This
            argument is applicable only when **definition** = HYDRAULIC and when **useBulkModulus** = True.
            Each sequence contains the following data:
            
            - The fluid bulk modulus.
            - Temperature, if the data depend on temperature.
            - Value of the first field variable, if the data depend on field variables.
            - Value of the second field variable.
            - Etc.
        useCapacity
            A Boolean specifying whether molar heat capacity values will be defined. This argument
            is applicable only when **definition** = PNEUMATIC. The default value is OFF.
        capacityType
            A SymbolicConstant specifying the method to define the molar heat capacity. Possible
            values are POLYNOMIAL and TABULAR. The default value is POLYNOMIAL.
        capacityTempDep
            A Boolean specifying whether the molar heat capacity data will have temperature
            dependency. This argument is applicable only when **definition** = PNEUMATIC, when
            **useCapacity** = True, and when **capacityType** = TABULAR. The default value is OFF.
        capacityDependencies
            An Int specifying the number of field variable dependencies in the molar heat capacity
            data. This argument is applicable only when **definition** = PNEUMATIC, when
            **useCapacity** = True, and when **capacityType** = TABULAR. The default value is 0.
        capacityTable
            A sequence of sequences of Floats specifying the molar heat capacity values in the form
            of a polynomial expression. This argument is applicable only when
            **definition** = PNEUMATIC, when **useCapacity** = True, and when **capacityType** = POLYNOMIAL. In
            this form, only one sequence is specified and that sequence contains the following data:
            
            - The first molar heat capacity coefficient.
            - The second molar heat capacity coefficient.
            - The third molar heat capacity coefficient.
            - The fourth molar heat capacity coefficient.
            - The fifth molar heat capacity coefficient.
            
            Alternatively, the sequence data may specify the molar heat capacity values at constant
            pressure for an ideal gas species. This argument is applicable only when
            **definition** = PNEUMATIC, when **useCapacity** = True, and when **capacityType** = TABULAR. Each
            sequence contains the following data:
            
            - The molar heat capacity at constant pressure.
            - Temperature, if the data depend on temperature.
            - Value of the first field variable, if the data depend on field variables.
            - Value of the second field variable.
            - Etc.

        Returns
        -------
        FluidCavityProperty
            A :py:class:`~abaqus.Interaction.FluidCavityProperty.FluidCavityProperty` object.
        """
        self.interactionProperties[name] = interactionProperty = FluidCavityProperty(
            name,
            definition,
            fluidDensity,
            molecularWeight,
            useExpansion,
            expansionTempDep,
            expansionDependencies,
            referenceTemperature,
            expansionTable,
            useBulkModulus,
            bulkModulusTempDep,
            bulkModulusDependencies,
            bulkModulusTable,
            useCapacity,
            capacityType,
            capacityTempDep,
            capacityDependencies,
            capacityTable,
        )
        return interactionProperty

    @abaqus_method_doc
    def FluidExchangeProperty(
        self,
        name: str,
        dataTable: tuple,
        definition: SymbolicConstant = BULK_VISCOSITY,
        pressureDependency: Boolean = OFF,
        temperatureDependency: Boolean = OFF,
        fieldDependencies: int = 0,
    ) -> FluidExchangeProperty:
        """This method creates a FluidExchangeProperty object.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].FluidExchangeProperty

        Parameters
        ----------
        name
            A String specifying the interaction property repository key.
        dataTable
            A sequence of sequences of Floats specifying the viscous and hydrodynamic resistance
            coefficients when **definition** = BULK_VISCOSITY. Each sequence contains the following
            data:
            
            - The viscous resistance coefficient.
            - The hydrodynamic resistance coefficient.
            - The average absolute pressure, if the data depend on pressure.
            - The average temperature, if the data depend on temperature.
            - The value of the first field variable, if the data depend on field variables.
            - The value of the second field variable.
            - Etc.
            
            Alternatively, the sequence data may specify the mass flow rate. This is applicable only
            when **definition** = MASS_FLUX. In this form, only one sequence is specified and that
            sequence contains the following data:
            
            - The mass flow rate per unit area.
            
            Alternatively, the sequence data may specify the mass rate leakage. This is applicable
            only when **definition** = MASS_RATE_LEAK. Each sequence contains the following data:
              
            - The absolute value of the mass flow rate per unit area. (The first tabular value
              entered must always be zero.)
            - The absolute value of the pressure difference. (The first tabular value entered must
              always be zero.)
            - The average absolute pressure, if the data depend on pressure.
            - The average temperature, if the data depend on temperature.
            - The value of the first field variable, if the data depend on field variables.
            - The value of the second field variable.
            - Etc.
            
            Alternatively, the sequence data may specify the volume flow rate. This is applicable
            only when **definition** = VOL_FLUX. In this form, only one sequence is specified and that
            sequence contains the following data:
            
            - The volumetric flow rate per unit area.
              
            Alternatively, the sequence data may specify the volume rate leakage. This is applicable
            only when **definition** = VOL_RATE_LEAK. Each sequence contains the following data:
            
            - The absolute value of the volumetric flow rate per unit area. (The first tabular value
              entered must always be zero.)
            - The absolute value of the pressure difference. (The first tabular value entered must
              always be zero.)
            - The average absolute pressure, if the data depend on pressure.
            - The average temperature, if the data depend on temperature.
            - The value of the first field variable, if the data depend on field variables.
            - The value of the second field variable.
            - Etc.
            
        definition
            A SymbolicConstant specifying the type of fluid exchange property to be defined.
            Possible values are BULK_VISCOSITY, MASS_FLUX, MASS_RATE_LEAK, VOL_FLUX, and
            VOL_RATE_LEAK. The default value is BULK_VISCOSITY.
        pressureDependency
            A Boolean specifying whether the data will have pressure dependency. This argument is
            applicable only when **definition** = BULK_VISCOSITY, or when **definition** = MASS_RATE_LEAK,
            or when **definition** = VOL_RATE_LEAK. The default value is OFF.
        temperatureDependency
            A Boolean specifying whether the data will have temperature dependency. This argument is
            applicable only when **definition** = BULK_VISCOSITY, or when **definition** = MASS_RATE_LEAK,
            or when **definition** = VOL_RATE_LEAK. The default value is OFF.
        fieldDependencies
            An Int specifying the number of field variable dependencies in the data. This argument
            is applicable only when **definition** = BULK_VISCOSITY, or when
            **definition** = MASS_RATE_LEAK, or when **definition** = VOL_RATE_LEAK. The default value is 0.

        Returns
        -------
        FluidExchangeProperty
            A :py:class:`~abaqus.Interaction.FluidExchangeProperty.FluidExchangeProperty` object.
        """
        self.interactionProperties[name] = interactionProperty = FluidExchangeProperty(
            name,
            dataTable,
            definition,
            pressureDependency,
            temperatureDependency,
            fieldDependencies,
        )
        return interactionProperty

    @abaqus_method_doc
    def FluidInflatorProperty(
        self,
        name: str,
        definition: str,
        effectiveArea: float,
        tankVolume: float,
        dischargeCoefficient: float = None,
        dataTable: tuple = (),
        numFluids: int = None,
        mixtureType: str = "",
        inflationTime: tuple = (),
        fluidbehaviorName: tuple = (),
        massFraction: tuple = (),
    ) -> FluidInflatorProperty:
        """This method creates a FluidInflatorProperty object.

        .. note::
            This function can be accessed by::

                mdb.models[name].FluidInflatorProperty

        .. versionadded:: 2019
            The `FluidInflatorProperty` method was added.

        Parameters
        ----------
        name
            A String specifying the interaction property repository key.
        definition
            A Symbolic constant specifying the method used for modeling the flow characteristics of
            inflators. The default value is **definition** = DUAL PRESSURE. The possible values are DUAL
            PRESSURE, PRESSURE AND MASS, TANK TEST, and TEMPERATURE AND MASS.
        effectiveArea
            A Float specifying the total inflator orifice area. This argument is applicable only if
            **definition** = DUAL PRESSURE or **definition** = PRESSURE AND MASS.
        tankVolume
            A Float specifying the tank volume. This argument is applicable only if
            **definition** = DUAL PRESSURE or **definition** = TANK TEST.
        dischargeCoefficient
            A Float specifying the discharge coefficient. This argument is applicable only if
            **definition** = DUAL PRESSURE or **definition** = PRESSURE AND MASS.
        dataTable
            A sequence of sequences of Floats specifying the items described in the "Table data"
            section below.
        numFluids
            An Int specifying the number of gas species used for this inflator.
        mixtureType
            A Symbolic constant specifying whether to use mass fraction or the molar fraction for a
            mixture of ideal gases. The default value is MASS FRACTION. The possible values are MASS
            FRACTION or MOLAR FRACTION.
        inflationTime
            A sequence of sequences of Floats specifying the inflation time.
        fluidbehaviorName
            A sequence of sequences of Strings specifying fluid behavior names.
        massFraction
            A sequence of sequences of Floats specifying the mass fraction or the molar fraction
            corresponding to entered fluid behavior.

        Returns
        -------
            A FluidInflatorProperty object.
        """
        self.interactionProperties[name] = interactionProperty = FluidInflatorProperty(
            name,
            definition,
            effectiveArea,
            tankVolume,
            dischargeCoefficient,
            dataTable,
            numFluids,
            mixtureType,
            inflationTime,
            fluidbehaviorName,
            massFraction,
        )
        return interactionProperty

    @abaqus_method_doc
    def IncidentWaveProperty(
        self,
        name: str,
        definition: SymbolicConstant = PLANAR,
        propagationModel: SymbolicConstant = ACOUSTIC,
        soundSpeed: float = None,
        fluidDensity: float = None,
        specificHeatRatio: float = None,
        gravity: float = None,
        atmosphericPressure: float = None,
        dragCoefficient: float = None,
        dragExponent: float = 2,
        waveEffects: Boolean = ON,
        chargeDensity: float = None,
        chargeMass: float = None,
        constantK1: float = None,
        constantK2: float = None,
        constantA: float = None,
        constantB: float = None,
        constantKc: float = None,
        duration: float = None,
        maximumSteps: int = 1500,
        relativeStepControl: float = None,
        absoluteStepControl: float = None,
        stepControlExponent: float = 0,
        genDecayA: float = 0,
        genDecayB: float = 0,
        genDecayC: float = 0,
        seedNumber: int = None,
        massTNT: float = None,
        massFactor: float = 1,
        lengthFactor: float = 1,
        timeFactor: float = 1,
        pressureFactor: float = 1,
    ) -> IncidentWaveProperty:
        """This method creates an IncidentWaveProperty object.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].IncidentWaveProperty

        Parameters
        ----------
        name
            A String specifying the interaction property repository key.
        definition
            A SymbolicConstant specifying the type of wave to be defined. Possible values are
            PLANAR, SPHERICAL, DIFFUSE, AIR_BLAST, and SURFACE_BLAST. The default value is PLANAR.
        propagationModel
            A SymbolicConstant specifying the spherical propagation model. Possible values are
            ACOUSTIC, UNDEX_CHARGE, and GENERALIZED_DECAY. The default value is ACOUSTIC.This
            argument is valid only when **definition** = SPHERICAL.
        soundSpeed
            A Float specifying the speed of sound in the fluid.This argument is not valid when
            **definition** = AIR_BLAST or when **definition** = SURFACE_BLAST.
        fluidDensity
            A Float specifying the fluid mass density.This argument is not valid when
            **definition** = AIR_BLAST or when **definition** = SURFACE_BLAST.
        specificHeatRatio
            None or a Float specifying the ratio of specific heats for gas. The default value is
            None.This argument is valid only when **definition** = SPHERICAL and
            **propagationModel** = UNDEX_CHARGE.
        gravity
            None or a Float specifying the acceleration due to gravity. The default value is
            None.This argument is valid only when **definition** = SPHERICAL and
            **propagationModel** = UNDEX_CHARGE.
        atmosphericPressure
            None or a Float specifying the atmospheric pressure. The default value is None.This
            argument is valid only when **definition** = SPHERICAL and **propagationModel** = UNDEX_CHARGE.
        dragCoefficient
            None or a Float specifying the fluid drag coefficient. The default value is None.This
            argument is valid only when **definition** = SPHERICAL and **propagationModel** = UNDEX_CHARGE.
        dragExponent
            A Float specifying the fluid drag exponent. The default value is 2.0.This argument is
            valid only when **definition** = SPHERICAL and **propagationModel** = UNDEX_CHARGE.
        waveEffects
            A Boolean specifying whether or not to include wave effects in the fluid and gas. The
            default value is ON.This argument is valid only when **definition** = SPHERICAL and
            **propagationModel** = UNDEX_CHARGE.
        chargeDensity
            None or a Float specifying the density of the charge material. The default value is
            None.This argument is valid only when **definition** = SPHERICAL and
            **propagationModel** = UNDEX_CHARGE.
        chargeMass
            None or a Float specifying the mass of the charge material. The default value is
            None.This argument is valid only when **definition** = SPHERICAL and
            **propagationModel** = UNDEX_CHARGE.
        constantK1
            None or a Float specifying the charge material constant K. The default value is
            None.This argument is valid only when **definition** = SPHERICAL and
            **propagationModel** = UNDEX_CHARGE.
        constantK2
            None or a Float specifying the charge material constant k. The default value is
            None.This argument is valid only when **definition** = SPHERICAL and
            **propagationModel** = UNDEX_CHARGE.
        constantA
            None or a Float specifying the charge material constant A. The default value is
            None.This argument is valid only when **definition** = SPHERICAL and
            **propagationModel** = UNDEX_CHARGE.
        constantB
            None or a Float specifying the charge material constant B. The default value is
            None.This argument is valid only when **definition** = SPHERICAL and
            **propagationModel** = UNDEX_CHARGE.
        constantKc
            None or a Float specifying the charge material constant Kc. The default value is
            None.This argument is valid only when **definition** = SPHERICAL and
            **propagationModel** = UNDEX_CHARGE.
        duration
            None or a Float specifying the time duration for the bubble simulation. The default
            value is None.This argument is valid only when **definition** = SPHERICAL and
            **propagationModel** = UNDEX_CHARGE.
        maximumSteps
            An Int specifying the maximum number of time steps for the bubble simulation. The
            default value is 1500.This argument is valid only when **definition** = SPHERICAL and
            **propagationModel** = UNDEX_CHARGE.
        relativeStepControl
            A Float specifying the relative step size control parameter. The default value is
            1x10-11.This argument is valid only when **definition** = SPHERICAL and
            **propagationModel** = UNDEX_CHARGE.
        absoluteStepControl
            A Float specifying the absolute step size control parameter. The default value is
            1x10-11.This argument is valid only when **definition** = SPHERICAL and
            **propagationModel** = UNDEX_CHARGE.
        stepControlExponent
            A Float specifying the step size control exponent. The default value is 0.2.This
            argument is valid only when **definition** = SPHERICAL and **propagationModel** = UNDEX_CHARGE.
        genDecayA
            A Float specifying the constant A associated with the generalized decay propagation
            model. The default value is 0.0.This argument is valid only when **definition** = SPHERICAL
            and **propagationModel** = GENERALIZED_DECAY.
        genDecayB
            A Float specifying the constant B associated with the generalized decay propagation
            model. The default value is 0.0.This argument is valid only when **definition** = SPHERICAL
            and **propagationModel** = GENERALIZED_DECAY.
        genDecayC
            A Float specifying the constant C associated with the generalized decay propagation
            model. The default value is 0.0.This argument is valid only when **definition** = SPHERICAL
            and **propagationModel** = GENERALIZED_DECAY.
        seedNumber
            An Int specifying the seed number (N) for the diffuse source calculation. N2 sources
            will be used in the simulation.This argument is valid only when **definition** = DIFFUSE.
        massTNT
            A Float specifying the equivalent mass of TNT, in any preferred mass unit.This argument
            is valid only when **definition** = AIR_BLAST or **definition** = SURFACE_BLAST.
        massFactor
            A Float specifying the multiplication factor to convert from the preferred mass unit to
            kilograms. The default value is 1.0.This argument is valid only when
            **definition** = AIR_BLAST or **definition** = SURFACE_BLAST.
        lengthFactor
            A Float specifying the multiplication factor to convert from the analysis length unit to
            meters. The default value is 1.0.This argument is valid only when **definition** = AIR_BLAST
            or **definition** = SURFACE_BLAST.
        timeFactor
            A Float specifying the multiplication factor to convert from the analysis time unit to
            seconds. The default value is 1.0.This argument is valid only when
            **definition** = AIR_BLAST or **definition** = SURFACE_BLAST.
        pressureFactor
            A Float specifying the multiplication factor to convert from the analysis pressure unit
            to pascals. The default value is 1.0.This argument is valid only when
            **definition** = AIR_BLAST or **definition** = SURFACE_BLAST.

        Returns
        -------
        IncidentWaveProperty
            An :py:class:`~abaqus.Interaction.IncidentWaveProperty.IncidentWaveProperty` object.
        """
        self.interactionProperties[name] = interactionProperty = IncidentWaveProperty(
            name,
            definition,
            propagationModel,
            soundSpeed,
            fluidDensity,
            specificHeatRatio,
            gravity,
            atmosphericPressure,
            dragCoefficient,
            dragExponent,
            waveEffects,
            chargeDensity,
            chargeMass,
            constantK1,
            constantK2,
            constantA,
            constantB,
            constantKc,
            duration,
            maximumSteps,
            relativeStepControl,
            absoluteStepControl,
            stepControlExponent,
            genDecayA,
            genDecayB,
            genDecayC,
            seedNumber,
            massTNT,
            massFactor,
            lengthFactor,
            timeFactor,
            pressureFactor,
        )
        return interactionProperty



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from ..UtilityAndView.abaqusConstants import *


@abaqus_class_doc
class TiffOptions:
    """The TiffOptions object stores the settings that Abaqus uses when printing in TIFF
    format. The TiffOptions object has no constructor. Abaqus creates the **tiffOptions**
    member when a session is started.

    .. note:: 
        This object can be accessed by::

            session.tiffOptions
    """

    @abaqus_method_doc
    def setValues(self, imageSize: SymbolicConstant = SIZE_ON_SCREEN):
        """This method modifies the TiffOptions object.

        Parameters
        ----------
        imageSize
            The SymbolicConstant SIZE_ON_SCREEN or a pair of Ints specifying the width and height of
            the image in pixels. Possible values are (**minWidth**, **minHeight**) ≤ **imageSize** ≤
            (**maxWidth** and **maxHeight**). The default value is SIZE_ON_SCREEN.
            Note:The minimum value of width and height (**minWidth** and **minHeight** ) is the number
            of pixels that occupy 10 mm at the current screen resolution. The value is typically
            around 50 pixels and may be different for width and height.
            The maximum value of width and height (**maxWidth** and **maxHeight** ) is the largest
            number of pixels supported by the system graphics and will be at least as large as the
            screen dimensions.

        Raises
        ------
        RangeError
            If either the width or height arguments of **imageSize** are out of range (where
            **minWidth** and **minHeight** are the number of pixels corresponding to approximately 10 mm
            for a given display and **maxWidth** and **maxHeight** are the largest allowable number of
            pixels supported by the system graphics).
        RangeError: imageSize must be SIZE_ON_SCREEN or a sequence of 2 Ints in the range
            (minWidth, minHeight) <= (width, height) <= (maxWidth, maxHeight).
        """
        ...



import typing

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from .ConstrainedSketchGeometry import ConstrainedSketchGeometry


@abaqus_class_doc
class EllipseByCenterPerimeter(ConstrainedSketchGeometry):
    @abaqus_method_doc
    def __init__(
        self, center: typing.Tuple[float, ...], axisPoint1: typing.Tuple[float, ...], axisPoint2: typing.Tuple[float, ...]
    ):
        """This method constructs an ellipse using a center point, a major axis point, and a minor
        axis point. The ellipse is added to the geometry repository of the ConstrainedSketch
        object.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].sketches[name].EllipseByCenterPerimeter

        Parameters
        ----------
        center
            A pair of Floats specifying the center point of the ellipse.
        axisPoint1
            A pair of Floats specifying the major or minor axis point of the ellipse.
        axisPoint2
            A pair of Floats specifying the minor or major axis point of the ellipse.

        Returns
        -------
        ConstrainedSketchGeometry
            A :py:class:`~abaqus.Sketcher.ConstrainedSketchGeometry.ConstrainedSketchGeometry.ConstrainedSketchGeometry` object (None if the ellipse cannot be created).

        """
        ...



import argparse
import os


def cli():
    parser = argparse.ArgumentParser(description='The abqpy command line interface')
    parser.add_argument('script', metavar='script', type=str, nargs=1,
                        help='the python script to run')
    parser.add_argument('-n', '--noGUI', dest='noGUI', action='store_true',
                        help='run abaqus command in batch mode')
    parser.add_argument('-m', '--mode', dest='mode', type=str, default='cae', choices=['cae', 'python'],
                        help='option to run Abaqus, either cae or python, by default cae')
    parser.add_argument('others', nargs=argparse.REMAINDER,
                        help='other arguments that will be passed to the abaqus command')
    args = parser.parse_args()
    if args.mode == 'cae':
        option = 'noGUI' if args.noGUI else 'script'
        cmd = f"abaqus cae {option}={args.script[0]} {' '.join(args.others)}"
        message = f'Running the following abaqus command: {cmd}'
        print('', '-' * len(message), message, '-' * len(message), sep='\n')
        os.system(cmd)
    elif args.mode == 'python':
        cmd = f"abaqus python {args.script} {' '.join(args.others)}"
        message = f'Running the following abaqus command: {cmd}'
        print('', '-' * len(message), message, '-' * len(message), sep='\n')
        os.system(cmd)
    else:
        print('Unknown mode, please use either cae or python')


if __name__ == '__main__':
    cli()



from abqpy.decorators import abaqus_class_doc
from .OdbInstance import OdbInstance
from .OdbPart import OdbPart
from .SectionPoint import SectionPoint
from ..UtilityAndView.abaqusConstants import *


@abaqus_class_doc
class FieldValue:
    """The FieldValue object represents the field data at a point. The FieldValue object has no
    constructor; it is created by the Odb object when data are added to the FieldOutput
    object using the addData method. For faster, bulk-data access, see Using bulk data
    access to an output database.

    .. note:: 
        This object can be accessed by::

            import odbAccess
            session.odbs[name].steps[name].frames[i].fieldOutputs[name].values[i]
    """

    #: A SymbolicConstant specifying the position of the output in the element. Possible values
    #: are:
    #: 
    #: - NODAL, specifying the values calculated at the nodes.
    #: - INTEGRATION_POINT, specifying the values calculated at the integration points.
    #: - ELEMENT_NODAL, specifying the values obtained by extrapolating results calculated at
    #:   the integration points.
    #: - ELEMENT_FACE, specifying the results obtained for surface variables such as cavity
    #:   radiation that are defined for the surface facets of an element.
    #: - CENTROID, specifying the value at the centroid obtained by extrapolating results
    #:   calculated at the integration points.
    position: SymbolicConstant = None

    #: A SymbolicConstant specifying the precision of the output in the element. Possible
    #: values are:
    #: 
    #: - SINGLE_PRECISION, specifying that the output values are in single precision.
    #: - DOUBLE_PRECISION, specifying that the output values are in double precision.
    precision: SymbolicConstant = None

    #: An Int specifying the element label of the element containing the location.
    #: **elementLabel** is available only if **position** = INTEGRATION_POINT, CENTROID,
    #: ELEMENT_NODAL, or ELEMENT_FACE.
    elementLabel: int = None

    #: An Int specifying the node label of the node containing the location. **nodelabel** is
    #: available only if **position** = ELEMENT_NODAL or NODAL.
    nodeLabel: int = None

    #: An Int specifying the integration point in the element. **integrationPoint** is available
    #: only if **position** = INTEGRATION_POINT.
    integrationPoint: int = None

    #: A SymbolicConstant specifying the face of the element. **face** is available only if
    #: **position** = ELEMENT_FACE.
    face: SymbolicConstant = None

    #: A SymbolicConstant specifying the output type. Possible values are SCALAR, VECTOR,
    #: TENSOR_3D_FULL, TENSOR_3D_PLANAR, TENSOR_3D_SURFACE, TENSOR_2D_PLANAR, and
    #: TENSOR_2D_SURFACE.
    type: SymbolicConstant = None

    #: A Float specifying the length or magnitude of the vector. **magnitude** is valid only when
    #: **type** = VECTOR.
    magnitude: float = None

    #: A Float specifying the calculated von Mises stress. The value is valid only when the
    #: **validInvariants** member includes MISES; otherwise, the value is indeterminate.
    #: Conjugate data will be ignored in invariant calculation.
    mises: float = None

    #: A Float specifying the calculated Tresca stress. The value is valid only when the
    #: **validInvariants** member includes TRESCA; otherwise, the value is indeterminate.
    #: Conjugate data will be ignored in invariant calculation.
    tresca: float = None

    #: A Float specifying the calculated pressure stress. The value is valid only when the
    #: **validInvariants** member includes PRESS; otherwise, the value is indeterminate.
    #: Conjugate data will be ignored in invariant calculation.
    press: float = None

    #: A Float specifying the calculated third stress invariant. The value is valid only when
    #: the **validInvariants** member includes INV3; otherwise, the value is indeterminate.
    #: Conjugate data will be ignored in invariant calculation.
    inv3: float = None

    #: A Float specifying the calculated maximum principal stress. The value is valid only when
    #: the **validInvariants** member includes MAX_PRINCIPAL; otherwise, the value is
    #: indeterminate. Conjugate data will be ignored in invariant calculation.
    maxPrincipal: float = None

    #: A Float specifying the calculated intermediate principal stress. The value is valid only
    #: when the **validInvariants** member includes MID_PRINCIPAL; otherwise, the value is
    #: indeterminate. Conjugate data will be ignored in invariant calculation.
    midPrincipal: float = None

    #: A Float specifying the minimum principal stress. The value is valid only when the
    #: **validInvariants** member includes MIN_PRINCIPAL; otherwise, the value is indeterminate.
    #: Conjugate data will be ignored in invariant calculation.
    minPrincipal: float = None

    #: A Float specifying the maximum principal in-plane stress. The value is valid only when
    #: the **validInvariants** member includes MAX_INPLANE_PRINCIPAL; otherwise, the value is
    #: indeterminate. Conjugate data will be ignored in invariant calculation.
    maxInPlanePrincipal: float = None

    #: A Float specifying the calculated minimum principal in-plane stress. The value is valid
    #: only when the **validInvariants** member includes MIN_INPLANE_PRINCIPAL; otherwise, the
    #: value is indeterminate. Conjugate data will be ignored in invariant calculation.
    minInPlanePrincipal: float = None

    #: A Float specifying the calculated principal out-of-plane stress. The value is valid only
    #: when the **validInvariants** member includes OUTOFPLANE_PRINCIPAL; otherwise, the value is
    #: indeterminate. Conjugate data will be ignored in invariant calculation.
    outOfPlanePrincipal: float = None

    #: An :py:class:`~abaqus.Odb.OdbInstance.OdbInstance` object specifying the part to which the labels belong.
    instance: OdbInstance = OdbInstance(
        "instance", OdbPart("part", THREE_D, DEFORMABLE_BODY)
    )

    #: A :py:class:`~abaqus.Odb.SectionPoint.SectionPoint` object.
    sectionPoint: SectionPoint = None

    #: A tuple of tuples of Floats specifying the 3 × 3 matrix of Floats specifying the
    #: direction cosines of the local coordinate system (the rotation from global to local).
    #: Each sequence represents a row in the direction cosine matrix. **localCoordSystem** is
    #: available for TENSOR data written in a local coordinate system. It is also available for
    #: VECTOR data for connector element outputs. For connector element outputs the rotation is
    #: from local to global. If the underlying data are in double precision, an exception will
    #: be thrown.
    localCoordSystem: tuple = ()

    #: A tuple of tuples of Floats specifying the 3 × 3 matrix of Doubles specifying the
    #: direction cosines of the local coordinate system (the rotation from global to local).
    #: Each sequence represents a row in the direction cosine matrix. **localCoordSystemDouble**
    #: is available for TENSOR data written in a local coordinate system. It is also available
    #: for VECTOR data for connector element outputs. For connector element outputs the
    #: rotation is from local to global. If the underlying data are in single precision, an
    #: exception will be thrown.
    localCoordSystemDouble: tuple = ()

    #: A tuple of Floats specifying data in the form described by **type**. If **type** = TENSOR or
    #: VECTOR, **data** is a sequence containing the components. If the underlying data are in
    #: double precision an exception will be thrown.
    data: tuple = ()

    #: A tuple of Floats specifying data in the form described by **type**. If **type** = TENSOR or
    #: VECTOR, **data** is a sequence containing the components. If the underlying data are in
    #: single precision, an exception will be thrown.
    dataDouble: tuple = ()

    #: A tuple of Floats specifying data in the form described by **type**. If **type** = TENSOR or
    #: VECTOR, **conjugateData** is a sequence containing the components. If the underlying data
    #: are in double precision, an exception will be thrown.
    conjugateData: tuple = ()

    #: A tuple of Floats specifying data in the form described by **type**. If **type** = TENSOR or
    #: VECTOR, **conjugateData** is a sequence containing the components. If the underlying data
    #: are in single precision, an exception will be thrown.
    conjugateDataDouble: tuple = ()
